/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#ifndef WSCOUT_GUI_FIND_DIALOG_HXX_
# define WSCOUT_GUI_FIND_DIALOG_HXX_

# include <QtGui/QApplication>

# include "wscout_gui_find_dialog.hh"
# include "wscout_hexdump.hh"

namespace wscout
{

  namespace gui
  {

    /*-----------.
    | FindThread |
    `-----------*/

    inline
    FindThread::FindThread(QObject*			parent,
			   const_iterator&		current_it,
			   const const_iterator&	last_it,
			   const std::string&		bytes,
			   size_t			first,
			   size_t			last):
      QThread (parent),
      current_it_ (current_it),
      last_it_ (last_it),
      first_ (first),
      last_ (last),
      canceled_ (false)
    {
      const std::string	prefix = bytes.substr(0, 2);
      const bool		has_prefix = prefix == "0x" or prefix == "0X";

      bytes_ = has_prefix?
	unhexdump(bytes.substr(2)):
	bytes_type (bytes.begin(), bytes.end());
    }

    inline
    FindThread::~FindThread()
    {
      cancel();
    }

    inline
    bool
    FindThread::canceled() const
    {
      return canceled_;
    }

    inline
    void
    FindThread::cancel()
    {
      canceled_ = true;
      wait();
    }

    /*-----------.
    | FindDialog |
    `-----------*/

    inline
    void
    FindDialog::voidIterators()
    {
      cancelThread();
      first_it_ = current_it_ = last_it_ = boost::none_t ();
    }

    inline
    void
    FindDialog::setIterators(const const_iterator&	first,
			     const const_iterator&	last,
			     size_t			last_index)
    {
      cancelThread();
      first_it_ = first;
      current_it_ = first;
      last_it_ = last;
      pbar_.setMaximum(last_index);
      pbar_.setValue(1);
    }

    inline
    void
    FindDialog::cancelThread()
    {
      if (threadRunning())
	thread_->cancel();
      qApp->processEvents(); // Call threadFinished().
    }

    inline
    bool
    FindDialog::threadRunning() const
    {
      return thread_.get() and not thread_->isFinished();
    }

    inline
    bool
    FindDialog::validIterators() const
    {
      return first_it_ and current_it_ and last_it_;
    }

  } // End of namespace gui.

} // End of namespace wscout.

#endif // ! WSCOUT_GUI_FIND_DIALOG_HXX_
