/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_LIST_OF_TRACES_HXX_
# define PCAP_LIST_OF_TRACES_HXX_

# include <boost/tokenizer.hpp>
# include <boost/foreach.hpp>

# include "list_of_traces.hh"

namespace pcapxx
{

  template <class Desc, class Bottom>
  list_of_traces<Desc, Bottom>::list_of_traces(const std::string& trace_names,
					       const std::string& separators)
  {
    typedef boost::char_separator<char>		separator_type;
    typedef boost::tokenizer<separator_type>	tokenizer_type;
    typedef std::vector<std::string>		vector;

    const separator_type	sep (separators.c_str());
    const tokenizer_type	tok (trace_names, sep);
    vector			names;

    BOOST_FOREACH(const std::string& t, tok)
      names.push_back(t);

    switch (names.size())
      {
      case 0:
	name_ = "[]";
	break;
      case 1:
	name_ = names.front();
	break;
      default:
	{
	  size_t	len = 0;
	  bool		cont = true;

	  while (cont)
	    {
	      const std::string	ref = names.front().substr(0, ++len);

	      for (size_t i = 1; cont and i < names.size(); ++i)
		if (names[i].substr(0, len) != ref)
		  cont = false;
	    }
	  name_ = '[' + names.front().substr(0, len - 1) + "*]";
	  break;
	}
      }

    BOOST_FOREACH(const std::string& s, names)
      push_back(Desc (s));
  }

  template <class Desc, class Bottom>
  const std::string&
  list_of_traces<Desc, Bottom>::file_name() const
  {
    return name_;
  }

  template <class Desc, class Bottom>
  void
  list_of_traces<Desc, Bottom>::expect(typename Desc::link_type l) const
  {
    for (size_t i = 0; i < this->size(); ++i)
      this->iterable(i).expect(l);
  }

} // End of namespace pcapxx.

#endif // ! PCAP_LIST_OF_TRACES_HXX_
