/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_FILTER_MICROSECONDS_STAMPER_HXX_
# define WIFI_FRAME_FILTER_MICROSECONDS_STAMPER_HXX_

# include "microseconds_stamper.hh"

namespace wifi
{
  namespace frame
  {
    namespace filter
    {

      namespace internals
      {

	template <class I, class HT, class B1, class B2>
	microseconds_stamper_iterator<I, HT, B1, B2>::
	microseconds_stamper_iterator(const iterable_type& i, bool end):
	  super_type (),
	  iterable_ (&i),
	  next_ (end ? i.last_ : i.first_)
	{
	  if (not end)
	    increment();
	}

	template <class I, class HT, class B1, class B2>
	bool
	microseconds_stamper_iterator<I, HT, B1, B2>::
	equal(const microseconds_stamper_iterator& rhs) const
	{
	  if (not rhs.value())
	    return not this->value();

	  const bool r (next_ == rhs.next_);

	  assert(not r or
		 tracker_.microseconds() == rhs.tracker_.microseconds());

	  return r;
	}

	template <class I, class HT, class B1, class B2>
	void
	microseconds_stamper_iterator<I, HT, B1, B2>::increment()
	{
	  using tool::endian::need_swap;

	  if (next_ == iterable_->last_)
	    {
	      this->value() = boost::none_t ();
	      return;
	    }

	  const pcapxx::pkthdr* p = next_->pcap_header().get();

	  tracker_.tick(reinterpret_cast<const HT*> (next_->bytes().get()),
			p->caplen, need_swap(iterable_->phy_end_, p->swapped));

	  this->value() = value_type (tracker_.microseconds(), *next_);
	  ++next_;
	}

      } // End of namespace internals.

      template <class I, class HT, class B>
      microseconds_stamper<I, HT, B>::
      microseconds_stamper(const I&			first,
			   const I&			last,
			   tool::endian::endianness	phy_end):
	first_ (first),
	last_ (last),
	phy_end_ (phy_end)
      {
      }

    } // End of namespace wifi::frame::filter.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_FILTER_MICROSECONDS_STAMPER_HXX_
