/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_TIME_CONVERTER_HXX_
# define WIFI_TIME_CONVERTER_HXX_

# include "time_converter.hh"

namespace wifi
{

  template <class Bottom>
  time_converter<Bottom>::
  time_converter(const pcaptime_type&		local_pcaptime,
		 const hosttime_type&		local_hosttime,
		 const mactime_type&		local_mactime,
		 const tool::microseconds&	global_time):
    super_type (),
    local_reference_ (local_pcaptime, local_hosttime, local_mactime),
    global_reference_ (global_time)
  {
  }

  template <class Bottom>
  time_converter<Bottom>::time_values::time_values()
  {
  }

  template <class Bottom>
  time_converter<Bottom>::
  time_values::time_values(const pcaptime_type&	pcaptime,
			   const hosttime_type&	hosttime,
			   const mactime_type&	mactime):
    pcaptime (pcaptime),
    hosttime (hosttime),
    mactime (mactime)
  {
  }

  template <class Bottom>
  typename time_converter<Bottom>::time_values
  time_converter<Bottom>::
  operator () (const tool::microseconds& global_time) const
  {
    enum { s = 1000000 }; // One second in microseconds.

    const mpz_class	dt	= global_time - global_reference_;

    mpz_class		usec	= local_reference_.pcaptime.tv_usec + dt;
    const mpz_class	sec	= local_reference_.pcaptime.tv_sec + usec / s;
    usec %= s;

    mpz_class		host	= local_reference_.hosttime + dt / 10;
    if (dt % 10 >= 5)
      ++host;

    const mpz_class	mac	= local_reference_.mactime + dt;

    time_values		r;

    r.pcaptime.tv_usec	= usec.get_ui();
    r.pcaptime.tv_sec	= sec.get_si();
    r.hosttime		= host.get_ui();
    r.mactime		= mac.get_ui();

    return r;
  }

} // End of namespace wifi.

#endif // ! WIFI_TIME_CONVERTER_HXX_
