/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <stdexcept>
#include <iostream>
#include <cstdlib>

#include "config.hh"

#include <wipal/tool/options.hh>
#include <wipal/pcap/list_of_traces.hh>
#include <wipal/pcap/dumper.hh>
#include <wipal/wifi/frame/filter/non_noisy_prism.hh>

int
main(int argc, char *const argv[])
{
  const tool::opt::options options =
    tool::opt::parse_args(argc, argv, wp_opt_prism_noise, 1, 1);

  try
    {
      typedef pcapxx::list_of_traces<>	list_of_traces;

      list_of_traces	traces (options.input[0]);

      if (traces.empty())
	throw std::invalid_argument ("Empty trace");

      pcapxx::dumper	dump = traces.iterable(0).dumper(options.output[0]);

      if (options.prism_noise_means_error)
	{
	  typedef list_of_traces::const_iterator		li;
	  typedef wifi::frame::filter::non_noisy_prism<li>	nnp;

	  traces.expect(pcapxx::descriptor<>::PRISM_HEADER);

	  nnp			filter (traces.begin(), traces.end());

	  filter.for_each(dump);
	}
      else
	{
	  traces.for_each(dump);
	}
    }
  catch (std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
